/*
 * Copyright (c) 2011-2014, Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#pragma once

#include "remote_processor_export.h"

#include "Message.h"
#include "RemoteCommand.h"
#include <vector>
#include <string>

class REMOTE_PROCESSOR_EXPORT CRequestMessage : public CMessage, public IRemoteCommand
{
public:
    CRequestMessage(const std::string &strCommand);
    CRequestMessage();

    // Command Name
    void setCommand(const std::string &strCommand);
    const std::string &getCommand() const override;

    // Arguments
    void addArgument(const std::string &strArgument) override;
    size_t getArgumentCount() const override;
    const std::string &getArgument(size_t argument) const override;
    const std::vector<std::string> &getArguments() const override;
    std::string packArguments(size_t startArgument, size_t nbArguments) const override;

private:
    /**
      * Constant character array.
      * This value defines the delimiters used to separate the arguments
      * in the request command.
      */
    static const char *const gacDelimiters;

    // Fill data to send
    void fillDataToSend() override;
    // Collect received data
    void collectReceivedData() override;
    // Size
    /**
     * @return size of the request message in bytes
     */
    size_t getDataSize() const override;
    // Trim input std::string
    static std::string trim(const std::string &strToTrim);

    // Command
    std::string _strCommand;
    // Arguments
    std::vector<std::string> _argumentVector;
};
